-- BountyHUD
-- By Pizzasgood (Rin Ganborro)
-- This software is released under the WTFPL Version 2 (http://sam.zoy.org/wtfpl/COPYING)
-- (In short:  Do whatever you want with it.)

BountyHUD = {}
BountyHUD.version = 1.0
BountyHUD.gui = {}
BountyHUD.gui.own_bounty = {}
BountyHUD.gui.own_bounty.box = nil
BountyHUD.gui.target_bounty = {}
BountyHUD.gui.target_bounty.box = nil

BountyHUD.active = gkini.ReadInt('BountyHUD', 'active', 0)
BountyHUD.own_bounty_active = gkini.ReadInt('BountyHUD', 'own_bounty', 1)
BountyHUD.target_bounty_active = gkini.ReadInt('BountyHUD', 'target_bounty', 1)


--acts like the real iup.Insert()
local function fake_iup_insert(ih, ref_child, new_child)
	--first add the new stuff
	iup.Append(ih, new_child)

	--now move to the end all the old stuff from ref_child on that is not new_child
	local last_child = ref_child
	local next_child = nil
	while true do
		if last_child ~= new_child then
			next_child = iup.GetNextChild(ih, last_child)
			iup.Detach(last_child)
			iup.Append(ih, last_child)
			last_child = next_child
		else
			break
		end
	end

	--and refresh it
	iup.Refresh(ih)
end


--include own bounty in selfinfo box
function BountyHUD:enable()
	self.active = 1
	gkini.WriteInt('BountyHUD', 'active', self.active)
	self:init()
end


--do not include own bounty in selfinfo box
function BountyHUD:disable()
	self.active = 0
	gkini.WriteInt('BountyHUD', 'active', self.active)
	self:init()
end


--toggle plugin
function BountyHUD:toggle()
	if self.active == 0 then
		self:enable()
	else
		self:disable()
	end
end


--include own bounty in selfinfo box
function BountyHUD:enable_own_bounty()
	self.own_bounty_active = 1
	gkini.WriteInt('BountyHUD', 'own_bounty', self.own_bounty_active)
	self:insert_own_bounty()
end


--do not include own bounty in selfinfo box
function BountyHUD:disable_own_bounty()
	self.own_bounty_active = 0
	gkini.WriteInt('BountyHUD', 'own_bounty', self.own_bounty_active)
	self:remove_own_bounty()
end


--toggle own bounty visibility
function BountyHUD:toggle_own_bounty()
	if self.own_bounty_active == 0 then
		self:enable_own_bounty()
	else
		self:disable_own_bounty()
	end
end


--add own bounty display to selfinfo box
function BountyHUD:insert_own_bounty()
	self:remove_own_bounty()

	local parent = HUD.selfinfo
	local first_child = iup.GetNextChild(parent)
	local reference = iup.GetBrother(iup.GetBrother(first_child))
	local title_font = first_child.font
	local bounty_font = first_child.font
	local title_color = first_child.fgcolor
	local bounty_color = iup.GetBrother(first_child).fgcolor

	--the title
	self.gui.own_bounty.title = iup.label { title = "Bounty", font = title_font, fgcolor = title_color, expand = "HORIZONTAL", alignment = "ALEFT" }
	--the value
	self.gui.own_bounty.bounty = iup.label { title = ' '..comma_value(self:get_bounty()), font = bounty_font, fgcolor = bounty_color, expand = "HORIZONTAL", alignment = "ALEFT" }
	--the container
	self.gui.own_bounty.box = iup.vbox {
		self.gui.own_bounty.title,
		self.gui.own_bounty.bounty;
		alignment = "ALEFT"
	}

	--add it to the HUD
	fake_iup_insert(parent, reference, self.gui.own_bounty.box)
	self.gui.own_bounty.box:show()

	--and refresh it
	iup.Refresh(parent)
end


--remove own bounty display from selfinfo box
function BountyHUD:remove_own_bounty()
	if self.gui.own_bounty.box ~= nil then
		iup.Destroy(self.gui.own_bounty.box)
		self.gui.own_bounty.box = nil
	end
end


--update own bounty display
function BountyHUD:update_own_bounty()
	if self.gui.own_bounty.box ~= nil then
		self.gui.own_bounty.bounty.title = ' '..comma_value(self:get_bounty())
	end
end


--return total bounty on player
function BountyHUD:get_bounty(charid)
	local bounty = 0
	local char_desc = GetCharacterDescription(charid)
	for line in string.gmatch(char_desc, 'placed a bounty on %w+ head of [%w ]*') do
		bounty = bounty + string.match(line, '%d+')
	end
	return bounty
end


--include target's bounty in targetframe box
function BountyHUD:enable_target_bounty()
	self.target_bounty_active = 1
	gkini.WriteInt('BountyHUD', 'target_bounty', self.target_bounty_active)
	self:insert_target_bounty()
end


--do not include target's bounty in targetframe box
function BountyHUD:disable_target_bounty()
	self.target_bounty_active = 0
	gkini.WriteInt('BountyHUD', 'target_bounty', self.target_bounty_active)
	self:remove_target_bounty()
end


--toggle target's bounty visibility
function BountyHUD:toggle_target_bounty()
	if self.target_bounty_active == 0 then
		self:enable_target_bounty()
	else
		self:disable_target_bounty()
	end
end


--add target's bounty display to targetframe box
function BountyHUD:insert_target_bounty()
	self:remove_target_bounty()

	local parent = HUD.targetframe[1]
	local distance = parent[3]
	local title_font = distance[1].font
	local bounty_font = distance[2].font
	local title_color = distance[1].fgcolor
	local bounty_color = distance[2].fgcolor

	--the title
	self.gui.target_bounty.title = iup.label { title = "Bounty:", font = title_font, fgcolor = title_color, expand = "NO", alignment = "ALEFT" }
	--the value
	self.gui.target_bounty.bounty = iup.label { title = comma_value(self:get_target_bounty()), font = bounty_font, fgcolor = bounty_color, expand = "HORIZONTAL", alignment = "ALEFT" }
	--the container
	self.gui.target_bounty.box = iup.hbox {
		self.gui.target_bounty.title,
		self.gui.target_bounty.bounty;
		alignment = "ATOP"
	}

	--add it to the HUD
	iup.Append(parent, self.gui.target_bounty.box)
	self.gui.target_bounty.box:show()

	--and refresh it
	iup.Refresh(parent)
end


--remove target's bounty display from selfinfo box
function BountyHUD:remove_target_bounty()
	if self.gui.target_bounty.box ~= nil then
		iup.Destroy(self.gui.target_bounty.box)
		self.gui.target_bounty.box = nil
	end
end


--update target's bounty display
function BountyHUD:update_target_bounty()
	if self.gui.target_bounty.box ~= nil then
		self.gui.target_bounty.bounty.title = comma_value(self:get_target_bounty())
	end
end


--return total bounty on target
function BountyHUD:get_target_bounty()
	local target_id = RequestTargetStats()
	if target_id ~= nil and target_id ~= GetCharacterID() then
		return self:get_bounty(target_id)
	else
		return 0
	end
end


--process events
function BountyHUD:OnEvent(e)
	if e == "HUD_SHOW" then
		self:init()
	elseif e == "UPDATE_CHARINFO" or e == "TARGET_CHANGED" or e == "PLAYER_STATS_UPDATED"then
		self:update_own_bounty()
		self:update_target_bounty()
	end
end


--initialize
function BountyHUD:init()
	if self.active == 1 then
		if self.own_bounty_active == 1 then
			self:insert_own_bounty()
		else
			self:remove_own_bounty()
		end
		if self.target_bounty_active == 1 then
			self:insert_target_bounty()
		else
			self:remove_target_bounty()
		end

		RegisterEvent(BountyHUD, "UPDATE_CHARINFO")
		RegisterEvent(BountyHUD, "TARGET_CHANGED")
		RegisterEvent(BountyHUD, "PLAYER_STATS_UPDATED")
	else
		self:remove_own_bounty()
		self:remove_target_bounty()

		UnregisterEvent(BountyHUD, "UPDATE_CHARINFO")
		UnregisterEvent(BountyHUD, "TARGET_CHANGED")
		UnregisterEvent(BountyHUD, "PLAYER_STATS_UPDATED")
	end
	UnregisterEvent(BountyHUD, "HUD_SHOW")
end


--display help
function BountyHUD:help()
	local command_color = "\127FFFFFF"
	local info_color = "\127D0D0D0"
	--print(command_color.."/bountyhud help")
	--print(info_color.."Displays this message")
	print(command_color.."/bountyhud [ on | off | toggle ]")
	print(info_color.."Turns the plugin on or off")
	print(command_color.."/bountyhud [ own | target ]")
	print(info_color.."Prints bounty information")
	print(command_color.."/bountyhud [ own_on | own_off | own_toggle ]")
	print(info_color.."Set whether your own bounty is shown on your hud")
	print(command_color.."/bountyhud [ target_on | target_off | target_toggle ]")
	print(info_color.."Set whether your target's bounty is shown on your hud")
end


--process commands
function BountyHUD:process(_,data)
	if (data ~= nil and data[1] == "on") then
		self:enable()
	elseif (data ~= nil and data[1] == "off") then
		self:disable()
	elseif (data ~= nil and data[1] == "toggle") then
		self:toggle()
	elseif (data ~= nil and data[1] == "own") then
		print(comma_value(self:get_bounty()))
	elseif (data ~= nil and data[1] == "own_on") then
		self:enable_own_bounty()
	elseif (data ~= nil and data[1] == "own_off") then
		self:disable_own_bounty()
	elseif (data ~= nil and data[1] == "own_toggle") then
		self:toggle_own_bounty()
	elseif (data ~= nil and data[1] == "target") then
		print(comma_value(self:get_target_bounty()))
	elseif (data ~= nil and data[1] == "target_on") then
		self:enable_target_bounty()
	elseif (data ~= nil and data[1] == "target_off") then
		self:disable_target_bounty()
	elseif (data ~= nil and data[1] == "target_toggle") then
		self:toggle_target_bounty()
	elseif (data ~= nil and data[1] == "help") then
		self:help()
	else
		self:help()
	end
end


RegisterUserCommand("bountyhud", function(_,data) BountyHUD:process(_, data) end)
RegisterEvent(BountyHUD, "HUD_SHOW")
